import 'package:flutter/material.dart';
import 'package:flutter_mobx/flutter_mobx.dart';
import 'package:flutter_modular/flutter_modular.dart';
import 'package:example_mvc_modular_complete/src/i18n/login_i18n.dart';
import 'package:example_mvc_modular_complete/src/shared/widgets/custom_alert_dialog/custom_alert_dialog.dart';
import 'package:example_mvc_modular_complete/src/shared/widgets/email_text_field/email_text_field_widget.dart';
import 'package:example_mvc_modular_complete/src/shared/widgets/password_text_field/password_text_field_widget.dart';

import 'register_controller.dart';

class RegisterPage extends StatefulWidget {
  const RegisterPage({Key key}) : super(key: key);

  @override
  _RegisterPageState createState() => _RegisterPageState();
}

class _RegisterPageState
    extends ModularState<RegisterPage, RegisterController> {
  //use 'controller' variable to access controller
  final _formKey = GlobalKey<FormState>();
  final _emailController = TextEditingController();
  final _passwordController = TextEditingController();

  Future<void> register() async {
    if (_formKey.currentState.validate()) {
      if (!await controller.registerWithEmail(
        _emailController.text,
        _passwordController.text,
      )) {
        CustomAlertDialog.error(context, "Erro no cadastro");
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text("Cadastro".i18n),
      ),
      body: Padding(
        padding: EdgeInsets.symmetric(horizontal: 16),
        child: Form(
          key: _formKey,
          child: Column(
            mainAxisAlignment: MainAxisAlignment.center,
            children: <Widget>[
              EmailTextFieldWidget(
                controller: _emailController,
                validationError: "E-mail inválido".i18n,
                label: "E-mail".i18n,
              ),
              PasswordTextFieldWidget(
                controller: _passwordController,
                validationError: "Senha inválida".i18n,
                label: "Senha".i18n,
              ),
              SizedBox(
                height: 25,
              ),
              Observer(builder: (_) {
                return RaisedButton(
                  onPressed: controller.loading ? null : register,
                  child: Text("Cadastrar-se".i18n),
                );
              }),
            ],
          ),
        ),
      ),
    );
  }
}
