import 'dart:io';

import 'package:flutter_snow_blower/flutter_snow_blower.dart';
import 'package:flutter_snow_blower/src/enums/architecture_enum.dart';
import 'package:flutter_snow_blower/src/utils/utils.dart' show mainDirectory;

Future<void> create({
  String projectName,
  String projectDescription,
  String projectOrg,
  bool isKotlin,
  bool isSwift,
  bool isAndroidX,
  Architecture architecture,
  bool complete,
  bool useEnvironment,
}) {
  return startFlutterCreate(
    projectName: projectName,
    projectDescription: projectDescription,
    projectOrg: projectOrg,
    isKotlin: isKotlin,
    isSwift: isSwift,
    isAndroidX: isAndroidX,
    architecture: architecture,
    complete: complete,
    useEnvironment: useEnvironment,
  );
}

Future<void> startFlutterCreate({
  String projectName,
  String projectDescription,
  String projectOrg,
  bool isKotlin,
  bool isSwift,
  bool isAndroidX,
  Architecture architecture,
  bool complete,
  bool useEnvironment,
}) {
  mainDirectory = projectName + '/';

  List<String> flutterArgs = createFlutterArgs(
    projectName,
    projectDescription,
    projectOrg,
    isKotlin,
    isSwift,
    isAndroidX,
  );

  return startSnowCreate(
    projectName,
    architecture,
    complete,
    useEnvironment,
    () => Process.start("flutter", flutterArgs, runInShell: true),
  );
}

Future<void> startSnowCreate(
  String projectName,
  Architecture architecture,
  bool complete,
  bool useEnvironment,
  Future<Process> Function() createProject,
) {
  return start(
    completeStart: complete,
    force: true,
    dir: Directory('$projectName/lib'),
    architecture: architecture,
    useEnviroment: useEnvironment,
    createProject: createProject,
  );
}

List<String> createFlutterArgs(
  String projectName,
  String projectDescription,
  String projectOrg,
  bool isKotlin,
  bool isSwift,
  bool isAndroidX,
) {
  projectDescription = projectDescription == null
      ? "A new Flutter project. Created by Snowman Labs with Flutter Snow Blower"
      : projectDescription;
  projectOrg = projectOrg == null ? "com.example" : projectOrg;

  var flutterArgs = ["create"];
  flutterArgs.add("--no-pub");

  if (isKotlin) {
    flutterArgs.add("-a");
    flutterArgs.add("kotlin");
  }

  if (isSwift) {
    flutterArgs.add("-i");
    flutterArgs.add("swift");
  }

  if (isAndroidX) {
    flutterArgs.add("--androidx");
  }

  flutterArgs.add("--project-name");
  flutterArgs.add("$projectName");

  if (projectDescription.isNotEmpty) {
    flutterArgs.add("--description");
    flutterArgs.add("'$projectDescription'");
  }

  if (projectOrg.isNotEmpty) {
    flutterArgs.add("--org");
    flutterArgs.add("$projectOrg");
  }

  flutterArgs.add(projectName);
  return flutterArgs;
}
