import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:flutter_snow_blower/flutter_snow_blower.dart';
import 'package:flutter_snow_blower/src/enums/architecture_enum.dart';
import 'package:flutter_snow_blower/src/modules/install.dart';
import 'package:flutter_snow_blower/src/modules/start/create/create.dart';
import 'package:flutter_snow_blower/src/modules/start/delete_lib.dart';
import 'package:flutter_snow_blower/src/modules/start/select_architecture.dart';
import 'package:flutter_snow_blower/src/utils/output_utils.dart' as output;
import 'package:flutter_snow_blower/src/utils/pubspec.dart';
import 'package:flutter_snow_blower/src/utils/utils.dart';

import 'change_min_sdk_version_build_gradle.dart';

part './install_packages.dart';

Future<void> start({
  bool completeStart,
  bool force = false,
  Directory dir,
  Architecture architecture,
  bool useEnviroment = true,
  Future<Process> Function() createProject,
}) async {
  dir ??= Directory('lib');
  String workingDirectory = dir.path.replaceAll("lib", "");

  architecture ??= await selectArchitecture();
  await deleteLib(dir, !dir.parent.existsSync());

  if (createProject != null) {
    final process = await createProject();
    stdout.addStream(process.stdout);
    stderr.addStream(process.stderr);
    final exitCode = await process.exitCode;
    if (exitCode != 0) {
      output.error("Falha na criação do projeto Flutter");
      exit(1);
    }
  }

  await deleteLib(dir, force);

  var dirTest = Directory(dir.parent.path + '/test');
  if (await dirTest.exists()) {
    if (dirTest.listSync().isNotEmpty) {
      output.msg("Removing test folder");
      await dirTest.delete(recursive: true);
    }
  }

  var package = await getNamePackage(dir.parent);

  await Future.wait<void>([
    _installPackages(
      dir.parent.path,
      completeStart,
      architecture,
      workingDirectory,
    ),
    createStartFiles(
      architecture,
      dir.path,
      completeStart,
      useEnviroment,
      package,
    ),
  ]);

  try {
    print("Replacing pubspec.yaml...");
    PubSpec().replacePubSpecYaml(workingDirectory);
  } on Exception {
    print("Replacing failed. Using the original version.");
  }

  await _getPackages(workingDirectory);
  await _buildRunnerBuild(workingDirectory);

  printSnowBlower();
  output.msg("\n\n            Project started! enjoy!");
}

Future<void> _getPackages(String workingDirectory) async {
  final finished = Completer<bool>();
  final process = await Process.start("flutter", ["packages", "get"],
      runInShell: true, workingDirectory: workingDirectory);
  process.stdout.transform(utf8.decoder).listen(
        print,
        cancelOnError: true,
        onDone: () => finished.complete(true),
        onError: (e) => finished.complete(true),
      );
  return finished.future;
}

Future<void> _buildRunnerBuild(String workingDirectory) async {
  final finished = Completer<bool>();
  final process = await Process.start(
      "flutter", ["pub", "run", "build_runner", "build"],
      runInShell: true, workingDirectory: workingDirectory);
  process.stdout.transform(utf8.decoder).listen(
        print,
        cancelOnError: true,
        onDone: () => finished.complete(true),
        onError: (e) => finished.complete(true),
      );
  return finished.future;
}
